import logging

from rest_framework import serializers

from core.models import ExamType, SubmissionType, Test, UserAccount
from util.factories import GradyUserFactory

from .generic import DynamicFieldsModelSerializer

log = logging.getLogger(__name__)
user_factory = GradyUserFactory()


class ExamSerializer(DynamicFieldsModelSerializer):

    class Meta:
        model = ExamType
        fields = ('pk', 'module_reference', 'total_score',
                  'pass_score', 'pass_only',)


class TestSerializer(DynamicFieldsModelSerializer):

    class Meta:
        model = Test
        fields = ('pk', 'name', 'label', 'annotation')


class SubmissionTypeListSerializer(DynamicFieldsModelSerializer):

    class Meta:
        model = SubmissionType
        fields = ('pk', 'name', 'full_score')


class SubmissionTypeSerializer(SubmissionTypeListSerializer):

    class Meta:
        model = SubmissionType
        fields = ('pk', 'name', 'full_score', 'description', 'solution')


class TutorSerializer(DynamicFieldsModelSerializer):
    done_assignments_count = serializers.IntegerField(
        read_only=True)

    def create(self, validated_data) -> UserAccount:
        log.info("Crating tutor from data %s", validated_data)
        return user_factory.make_tutor(
            username=validated_data['username'])

    class Meta:
        model = UserAccount
        fields = ('pk', 'username', 'done_assignments_count')
