from rest_framework import serializers

from core.models import Submission
from core.serializers import (DynamicFieldsModelSerializer, FeedbackSerializer,
                              SubmissionTypeListSerializer,
                              SubmissionTypeSerializer, TestSerializer)


class SubmissionNoTextFieldsSerializer(DynamicFieldsModelSerializer):
    score = serializers.ReadOnlyField(source='feedback.score')
    type = serializers.ReadOnlyField(source='type.name')
    full_score = serializers.ReadOnlyField(source='type.full_score')

    class Meta:
        model = Submission
        fields = ('pk', 'type', 'score', 'full_score')


class SubmissionSerializer(DynamicFieldsModelSerializer):
    type = SubmissionTypeSerializer()
    feedback = FeedbackSerializer()
    tests = TestSerializer(many=True)

    class Meta:
        model = Submission
        fields = ('pk', 'type', 'text', 'feedback', 'tests')


class SubmissionListSerializer(DynamicFieldsModelSerializer):
    type = SubmissionTypeListSerializer(fields=('pk', 'name', 'full_score'))
    # TODO change this according to new feedback model
    feedback = FeedbackSerializer()

    class Meta:
        model = Submission
        fields = ('pk', 'type', 'feedback')
