import {mut} from './mutations'
import {authMut} from '@/store/modules/authentication'
import * as api from '@/api'
import router from '@/router/index'

function passErrorIfResponsePresent (err, fallbackMsg) {
  if (err.response) {
    throw err
  } else {
    if (fallbackMsg) {
      throw new Error(fallbackMsg)
    }
  }
}

const actions = {
  async getSubscriptions ({ commit }) {
    try {
      const subscriptions = await api.fetchSubscriptions()
      commit(mut.SET_SUBSCRIPTIONS, subscriptions)
    } catch (err) {
      passErrorIfResponsePresent(err, 'Unable to fetch subscriptions')
    }
  },
  async getExamTypes ({commit}) {
    try {
      const examTypes = await api.fetchExamType({})
      commit(mut.SET_EXAM_TYPES, examTypes)
    } catch (err) {
      passErrorIfResponsePresent(err, 'Unable to fetch exam mut')
    }
  },
  async subscribeTo ({ commit }, {type, key, stage}) {
    try {
      const subscription = await api.subscribeTo(type, key, stage)
      commit(mut.SET_SUBSCRIPTION, subscription)
    } catch (err) {
      passErrorIfResponsePresent(err, 'Subscribing unsuccessful')
    }
  },
  async updateSubmissionTypes ({ commit }, fields) {
    try {
      const submissionTypes = await api.fetchSubmissionTypes(fields)
      submissionTypes.forEach(type => {
        commit(mut.UPDATE_SUBMISSION_TYPE, type)
      })
    } catch (err) {
      passErrorIfResponsePresent(err)
    }
  },
  async getCurrentAssignment ({ commit }, subscriptionPk) {
    try {
      const assignment = await api.fetchCurrentAssignment(subscriptionPk)
      commit(mut.UPDATE_ASSIGNMENT, {
        assignment,
        subscriptionPk,
        key: 'currentAssignment'
      })
      return assignment
    } catch (err) {
      passErrorIfResponsePresent(err, "Couldn't fetch assignment")
    }
  },
  async getNextAssignment ({ commit }, subscriptionPk) {
    try {
      const assignment = await api.fetchNextAssignment(subscriptionPk)
      commit(mut.UPDATE_ASSIGNMENT, {
        assignment,
        subscriptionPk,
        key: 'nextAssignment'
      })
      return assignment
    } catch (err) {
      passErrorIfResponsePresent(err, "Couldn't fetch assignment")
    }
  },
  logout ({ commit }, message = '') {
    commit(mut.RESET_STATE)
    commit(authMut.SET_MESSAGE, message)
    router.push('/')
  }
}

export default actions
