import {fetchStudentSelfData, fetchStudentSubmissions} from '../../api'

export const studentPageMut = Object.freeze({
  SET_STUDENT_NAME: 'SET_STUDENT_NAME',
  SET_EXAM: 'SET_EXAM',
  SET_SUBMISSION_TYPES: 'SET_SUBMISSION_TYPES',
  SET_SUBMISSIONS_FOR_LIST: 'SET_SUBMISSIONS_FOR_LIST',
  SET_FULL_SUBMISSION_DATA: 'SET_FULL_SUBMISSION_DATA',
  SET_VISITED: 'SET_VISITED',
  SET_LOADED: 'SET_LOADED'
})

const studentPage = {
  state: {
    studentName: '',
    exam: {},
    submissionsForList: [],
    submissionData: {},
    visited: {},
    loaded: false
  },
  mutations: {
    [studentPageMut.SET_STUDENT_NAME]: function (state, name) {
      state.studentName = name
    },
    [studentPageMut.SET_EXAM]: function (state, exam) {
      state.exam = exam
    },
    [studentPageMut.SET_SUBMISSION_TYPES]: function (state, submissionTypes) {
      state.submissionTypes = submissionTypes
    },
    [studentPageMut.SET_SUBMISSIONS_FOR_LIST]: function (state, submissions) {
      state.submissionsForList = submissions
    },
    /**
     * Reduces the array submissionData returned by the /api/student-submissions
     * into an object where the keys are the SubmissionType id's and the values
     * the former array elements. This is done to have direct access to the data
     * via the SubmissionType id.
     */
    [studentPageMut.SET_FULL_SUBMISSION_DATA]: function (state, submissionData) {
      state.submissionData = submissionData.reduce((acc, cur, index) => {
        acc[cur.type.pk] = cur
        return acc
      }, {})
    },
    [studentPageMut.SET_VISITED]: function (state, visited) {
      state.visited = { ...state.visited, [visited.index]: visited.visited }
    },
    [studentPageMut.SET_LOADED]: function (state, loaded) {
      state.loaded = loaded
    }
  },
  actions: {

    async getStudentData (context) {
      try {
        const studentData = await fetchStudentSelfData()
        context.commit(studentData.SET_STUDENT_NAME, studentData.name)
        context.commit(studentData.SET_EXAM, studentData.exam)
        context.commit(studentData.SET_SUBMISSIONS_FOR_LIST, studentData.submissions)
        context.commit(studentData.SET_LOADED, true)
      } catch (e) {
        this.$notify({
          title: 'API Fail',
          text: 'Unable to fetch student data',
          type: 'error'
        })
        console.log(e)
      }
    },

    async getStudentSubmissions (context) {
      try {
        const submissions = await fetchStudentSubmissions()
        context.commit(studentPageMut.SET_FULL_SUBMISSION_DATA, submissions)
      } catch (e) {
        this.$notify({
          title: 'API Fail',
          text: 'Unable to fetch student submissions',
          type: 'error'
        })
      }
    }
  }
}

export default studentPage
